<?php
/*
 * --------------------------------------------------------------------------------
   شرکت پرداخت الکترونیک پاسارگاد
 * --------------------------------------------------------------------------------
 * @subpackage	درگاه پرداخت پاسارگاد برای osCommerce
 * @author    	شرکت پرداخت الکترونیک پاسارگاد - https://pep.co.ir
 * @link		https://pep.co.ir
 * @Copyright	تمامی حقوق مادی و معنوی برای شرکت پرداخت الکترونیک پاسارگاد محفوظ می باشد.
 * --------------------------------------------------------------------------------
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class pasargad extends PaymentModule
{

    private $_html = '';
    private $_postErrors = array();

    public function __construct()
    {
        $this->name = 'pasargad';
        $this->tab = 'payments_gateways';
        $this->version = '1.0';
        $this->author = 'شرکت پرداخت الکترونیک پاسارگاد';

        $this->currencies = true;
        $this->currencies_mode = 'radio';

        parent::__construct();

        $this->page = basename(__FILE__, '.php');
        $this->displayName = $this->l('درگاه پرداخت پاسارگاد');
        $this->description = $this->l('ماژول درگاه امن پاسارگاد برای پرستاشاپ');
        $this->confirmUninstall = $this->l('آیا از حذف اطمینان دارید ؟');

        if (!sizeof(Currency::checkPaymentCurrencies($this->id)))
            $this->warning = $this->l('No currency has been set for this module');

        $config = Configuration::getMultiple(array('pasargad_terminal_id', ''));
        if (!isset($config['pasargad_terminal_id']))
            $this->warning = $this->l('Your Pasargad Terminal ID must be configured in order to use this module');

        $config = Configuration::getMultiple(array('pasargad_merchant_id', ''));
        if (!isset($config['pasargad_merchant_id']))
            $this->warning = $this->l('Your Pasargad Merchant ID must be configured in order to use this module');

        if ($_SERVER['SERVER_NAME'] == 'localhost')
            $this->warning = $this->l('Your are in localhost, Pasargad can\'t validate order');
    }

    public function install()
    {
        if (!parent::install()
            OR !Configuration::updateValue('pasargad_terminal_id', '')
            OR !Configuration::updateValue('pasargad_merchant_id', '')
            OR !$this->registerHook('payment')
            OR !$this->registerHook('paymentReturn')) {
            return false;
        } else {
            return true;
        }
    }

    public function uninstall()
    {
        if (!Configuration::deleteByName('pasargad_terminal_id')
            OR !Configuration::deleteByName('pasargad_merchant_id')
            OR !parent::uninstall())
            return false;
        return true;
    }

    public function displayFormSettings()
    {
        $this->_html .= '
		<form action="' . $_SERVER['REQUEST_URI'] . '" method="post">
			<fieldset>
				<legend><img src="../img/admin/cog.gif" alt="" class="middle" />تنظیمات درگاه امن پاسارگاد</legend>
				<label>شماره ترمینال :</label>
				<div class="margin-form"><input type="text" size="30" name="pasargad_terminal_id" value="' . Configuration::get('pasargad_terminal_id') . '" /></div>			
				<label>شماره فروشگاه :</label>
				<div class="margin-form"><input type="text" size="30" name="pasargad_merchant_id" value="' . Configuration::get('pasargad_merchant_id') . '" /></div>
				<center><input type="submit" name="submitPasargad" value="بروزرسانی تنظیمات" class="button" /></center>			
			</fieldset>
		</form>';
    }

    public function displayConf()
    {
        $this->_html .= '<div class="bootstrap">
		<div class="alert alert-success">
			<button type="button" class="close" data-dismiss="alert">×</button>
			تنظیمات درگاه پرداخت پاسارگاد با موفقیت انجام شد.
		</div>
	</div>';
    }

    public function displayErrors()
    {
        foreach ($this->_postErrors AS $err)
            $this->_html .= '<div class="bootstrap">
		<div class="alert alert-danger">
			<button type="button" class="close" data-dismiss="alert">×</button>
			' . $err . '
		</div>
	</div>';
    }

    public function getContent()
    {
        $this->_html = '<h2>' . $this->l('Pasargad Payment') . '</h2>';
        if (isset($_POST['submitPasargad'])) {
            if (empty($_POST['pasargad_terminal_id']))
                $this->_postErrors[] = 'شماره ترمینال را وارد کنید.';
            if (empty($_POST['pasargad_merchant_id']))
                $this->_postErrors[] = 'شماره فروشگاه را وارد کنید.';
            if (!sizeof($this->_postErrors)) {
                Configuration::updateValue('pasargad_terminal_id', $_POST['pasargad_terminal_id']);
                Configuration::updateValue('pasargad_merchant_id', $_POST['pasargad_merchant_id']);
                $this->displayConf();
            } else
                $this->displayErrors();
        }

        $this->displayFormSettings();
        return $this->_html;
    }

    private function displayPasargad()
    {
        $this->_html .= '<img src="../modules/pasargad/pasargad.jpg" style="float:left; margin-right:15px;"><b>' . $this->l('This module allows you to accept payments by Pasargad.') . '</b><br /><br />
		' . $this->l('Any cart from Shetab Banks are accepted.') . '<br /><br /><br />';

    }

    public function execPayment($cart)
    {
        global $cookie, $smarty;
        $purchase_currency = $this->GetCurrency();
        if ($cookie->id_currency == $purchase_currency->id)
            $PurchaseAmount = number_format($cart->getOrderTotal(true, 3), 0, '', '');
        else
            $PurchaseAmount = number_format(Tools::convertPrice($cart->getOrderTotal(true, 3), $purchase_currency), 0, '', '');
        $PurchaseAmount *= 1;

        $order_id = $cart->id . mt_rand(10, 99);
        $TerminalID = Configuration::get('pasargad_terminal_id');
        $MerchantID = Configuration::get('pasargad_merchant_id');
        $amount = intval($PurchaseAmount);
        $CallBackUrl = (Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://') . $_SERVER['HTTP_HOST'] . __PS_BASE_URI__ . 'modules/pasargad/validation.php?oid=' . $order_id;
        $Request = self::PepPayRequest($order_id, $TerminalID, $MerchantID, $amount, $CallBackUrl);
        if (isset($Request) && $Request->IsSuccess) {
            self::d_redirect('https://pep.shaparak.ir/payment.aspx?n='.$Request->Token);
        }
        else {
            self::display_error('', '', $order_id, 0, isset($Request->Message) ? $Request->Message : 'خطای نامشخص');
        }
    }

    public static function d_redirect($url = '')
    {
        if ($url != '') {
            if (headers_sent()) {
                echo '<script type="text/javascript">window.location.assign("' . $url . '")</script>';
            } else {
                header("Location: $url");
            }
            exit();
        }
    }

    public static function PepPayRequest($InvoiceNumber, $TerminalCode, $MerchantCode, $Amount, $RedirectAddress, $Mobile = '', $Email = '')
    {
        require_once(dirname(__FILE__) . '/includes/RSAProcessor.class.php');
        $processor = new RSAProcessor(dirname(__FILE__) . '/includes/certificate.xml', RSAKeyType::XMLFile);
        if (!function_exists('jdate')) {
            require_once(dirname(__FILE__) . '/includes/jdf.php');
        }
        $data = array(
            'InvoiceNumber' => $InvoiceNumber,
            'InvoiceDate' => jdate('Y/m/d'),
            'TerminalCode' => $TerminalCode,
            'MerchantCode' => $MerchantCode,
            'Amount' => $Amount,
            'RedirectAddress' => $RedirectAddress,
            'Timestamp' => date('Y/m/d H:i:s'),
            'Action' => 1003,
            'Mobile' => $Mobile,
            'Email' => $Email
        );

        $sign_data = json_encode($data);
        $sign_data = sha1($sign_data, true);
        $sign_data = $processor->sign($sign_data);
        $sign = base64_encode($sign_data);

        $curl = curl_init('https://pep.shaparak.ir/Api/v1/Payment/GetToken');
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
                'Sign: ' . $sign
            )
        );
        $result = json_decode(curl_exec($curl));
        curl_close($curl);

        return $result;
    }

    public static function display_error($pay_status = '', $tran_id = '', $order_id = '', $is_callback = 1, $message)
    {
        if ($pay_status == 'retry') {
            $page_title = 'خطای موقت در پرداخت';
            $admin_mess = 'در هنگام بازگشت خریدار از بانک سرور بانک پاسخ نداد ، از خریدار درخواست شد صفحه را رفرش کند';
            $retry_mess = '
				<div style="margin:15px 0px 21px 0px;font-size: 12px;">
					سرور درگاه اینترنتی <span style="color:#ff0000;">به صورت موقت</span> با مشکل مواجه شده است ، جهت تکمیل تراکنش لحظاتی بعد بر روی دکمه زیر کلیک کنید
				</div>
				<div style="margin:20px 0px 25px 0px;color:#008800;" id="reqreload">
					<button onclick="reload_page()">تلاش مجدد</button>
				</div>
				<script>
					function reload_page(){
						document.getElementById("reqreload").innerHTML = "در حال تلاش مجدد لطفا صبر کنید ..";
						location.reload();
					}
				</script>';
        } elseif ($pay_status == 'reversal_done') {
            $page_title = 'مشکل در ارائه خدمات';
            $admin_mess = 'خریدار مبلغ را پرداخت کرد اما در هنگام بازگشت از بانک مشکلی در ارائه خدمات رخ داد ، دستور بازگشت وجه به حساب خریدار در بانک ثبت شد';
            $client_mess = 'پرداخت شما با شماره پیگیری ' . $tran_id . ' با موفقیت در بانک انجام شده است اما در ارائه خدمات مشکلی رخ داده است !<br />دستور بازگشت وجه به حساب شما در بانک ثبت شده است ، در صورتی که وجه پرداختی تا ساعات آینده به حساب شما بازگشت داده نشد با پشتیبانی تماس بگیرید (نهایت مدت زمان بازگشت به حساب 72 ساعت می باشد)';
        } elseif ($pay_status == 'reversal_error') {
            $page_title = 'مشکل در ارائه خدمات';
            $admin_mess = 'خریدار مبلغ را پرداخت کرد اما در هنگام بازگشت از بانک مشکلی در ارائه خدمات رخ داد ، دستور بازگشت وجه به حساب خریدار در بانک ثبت شد اما متاسفانه با خطا روبرو شد ، به این خریدار باید یا خدمات ارائه شود یا وجه استرداد گردد';
            $client_mess = 'پرداخت شما با شماره پیگیری ' . $tran_id . ' با موفقیت در بانک انجام شده است اما در ارائه خدمات مشکلی رخ داده است !<br />به منظور ثبت دستور بازگشت وجه به حساب شما در بانک اقدام شد اما متاسفانه با خطا روبرو شد ، لطفا به منظور دریافت خدمات و یا استرداد وجه پرداختی با پشتیبانی تماس بگیرید';
        } elseif ($pay_status == 'order_not_exist') {
            $page_title = 'سفارش یافت نشد';
            $admin_mess = 'سفارش در سایت یافت نشد';
            $client_mess = 'متاسفانه سفارش شما در سایت یافت نشد ! در صورتی که وجه پرداختی از حساب بانکی شما کسر شده باشد به صورت خودکار از سوی بانک به حساب شما باز خواهد گشت (نهایت مدت زمان بازگشت به حساب 72 ساعت می باشد)';
        } elseif ($pay_status == 'invoice_id_is_blank') {
            $page_title = 'خطا در پارامتر ورودی';
            $admin_mess = 'پس از بازگشت از بانک شماره سفارش موجود نبود';
            $client_mess = 'متاسفانه پارامتر ورودی شما معتبر نیست ! در صورتی که وجه پرداختی از حساب بانکی شما کسر شده باشد به صورت خودکار از سوی بانک به حساب شما باز خواهد گشت (نهایت مدت زمان بازگشت به حساب 72 ساعت می باشد)';
        } elseif ($pay_status == 'already_been_completed') {
            $page_title = 'سفارش پیش از این موفق شده است';
            $client_mess = 'سفارش شما پیش از این با شماره پیگیری ' . $tran_id . ' با موفقیت ثبت شده است';
        } elseif ($pay_status == 'order_not_for_this_person') {
            $page_title = $admin_mess = 'شماره سفارش نادرست است';
            $client_mess = 'شماره سفارش نادرست است ؛ در صورت نیاز به پشتیبانی تماس بگیرید';
        } elseif ($is_callback == 0) {
            $page_title = 'خطا در ارسال به بانک';
            $admin_mess = 'خطای  در ارسال به بانک';
            $client_mess = '<p style="text-align:center;">' . $message . '</p>';
        } else {
            $page_title = $admin_mess = 'پرداخت انجام نشد';
            $client_mess = $message;
            $client_mess .= ' ؛ در صورتی که وجه پرداختی از حساب بانکی شما کسر شده باشد به صورت خودکار از سوی بانک به حساب شما باز خواهد گشت (نهایت مدت زمان بازگشت به حساب 72 ساعت می باشد)';
        }

        echo '
			<div align="center" dir="rtl" style="font-family:tahoma;font-size:12px;border:1px dotted #c3c3c3; width:60%; margin: 50px auto 0px auto;line-height: 25px;padding-left: 12px;padding-top: 8px;">
				<span style="color:#ff0000;"><b>' . $page_title . '</b></span><br/><p style="text-align:right;margin-right:8px;">' . $client_mess . '</p><a href="../../order">بازگشت >></a><br/><br/>
			</div>';
        exit();
    }

    public function hookPayment($params)
    {
        if (!$this->active)
            return;

        return $this->display(__FILE__, 'payment.tpl');
    }

    public function hookPaymentReturn($params)
    {
        if (!$this->active)
            return;

        return $this->display(__FILE__, 'confirmation.tpl');
    }

}  